Index: ip_output.c
===================================================================
RCS file: /home/ncvs/src/sys/netinet/ip_output.c,v
retrieving revision 1.217
diff -u -p -r1.217 ip_output.c
--- ip_output.c	2 May 2004 15:10:16 -0000	1.217
+++ ip_output.c	4 May 2004 09:16:30 -0000
@@ -282,6 +282,12 @@ ip_output(struct mbuf *m, struct mbuf *o
 			error = EHOSTUNREACH;
 			goto bad;
 		}
+		if ((ro->ro_rt->rt_flags & RTF_REJECT) &&
+		     ro->ro_rt->rt_rmx.rmx_expire >= time_second) {
+			ipstat.ips_odropped++;
+			error = EHOSTDOWN;
+			goto bad;
+		}
 		ia = ifatoia(ro->ro_rt->rt_ifa);
 		ifp = ro->ro_rt->rt_ifp;
 		ro->ro_rt->rt_rmx.rmx_pksent++;
@@ -404,15 +410,22 @@ ip_output(struct mbuf *m, struct mbuf *o
 		}
 	}
 #endif /* notdef */
+
+	/* Verify that the interface is ready */
+	if ((ifp->if_flags & (IFF_UP|IFF_RUNNING)) != (IFF_UP|IFF_RUNNING)) {
+		error = ENETDOWN;
+		ipstat.ips_odropped++;
+		goto bad;
+	}
 	/*
 	 * Verify that we have any chance at all of being able to queue
 	 *      the packet or packet fragments
 	 */
 	if ((ifp->if_snd.ifq_len + ip->ip_len / ifp->if_mtu + 1) >=
-		ifp->if_snd.ifq_maxlen) {
-			error = ENOBUFS;
-			ipstat.ips_odropped++;
-			goto bad;
+	     ifp->if_snd.ifq_maxlen) {
+		error = ENOBUFS;
+		ipstat.ips_odropped++;
+		goto bad;
 	}
 
 	/*
@@ -1077,6 +1101,10 @@ bad:
  *
  * if_hwassist_flags is the hw offload capabilities (see if_data.ifi_hwassist)
  * sw_csum contains the delayed checksums flags (e.g., CSUM_DELAY_IP).
+ *
+ * Note: ip_fragment expects ip_len and ip_off in the supplied mbuf to be in
+ * host byte order.  However the returned fragment chain will be entirely in
+ * network byte order.
  */
 int
 ip_fragment(struct ip *ip, struct mbuf **m_frag, int mtu,
